﻿using numl.Math.LinearAlgebra;
using numl.Supervised.Regression;
using System;

namespace LRExample
{
    class Program
    {
        static void Main(string[] args)
        {
            // number of training examples (200 books with ratings)
            var n = 200;

            // number of features (categories according to which books are rated)
            var m = 5;

            // initialize feature matrix (how each film is weighted by category)
            // and label vector (how one specific user rated each film)
            // with random values
            Matrix x = new Matrix(n, m);
            Vector y = new Vector(n);
            Random rnd = new Random();
            for (int i = 0; i < n; i++)
            {
                // ratings are in range 0-5
                y[i] = rnd.Next(0, m);

                // feature weights are in range 0-1000
                for (var j = 0; j < m; j++)
                {
                    x[i, j] = rnd.Next(0, 1000);
                }
            };

            // create supervised learning model that uses linear regression algorithm
            LinearRegressionGenerator generator = new LinearRegressionGenerator()
            {
                LearningRate = 0.1,     // big steps - fast but may diverge; small steps - may run too slowly
                MaxIterations = 400,    // how many times do you want it to run
                Lambda = 0              // compensation factor to prevent overfitting of training data - just ignore it for now
            };

            // now train the model - this is where all the machine learning is happening
            LinearRegressionModel model = (LinearRegressionModel)generator.Generate(x.Copy(), y.Copy());

            // display optimal weight vector for our user - this is the final model
            Console.WriteLine(model.Theta);
            Console.ReadKey();

            // now let's create a bunch of unrated books,
            // and for each one, we'll predict the rating
            Matrix x_unrated = new Matrix(n, m);
            Vector y_prediction = new Vector(n);
            for (int i = 0; i < n; i++)
            {
                // feature weights are in range 0-1000
                for (var j = 0; j < m; j++)
                {
                    x_unrated[i, j] = rnd.Next(0, 1000);
                }

                // based on these weights, make a prediction for the rating
                var unratedBook = new Vector(new double[] { 
                    x_unrated[i, 0], 
                    x_unrated[i, 1], 
                    x_unrated[i, 2], 
                    x_unrated[i, 3], 
                    x_unrated[i, 4] 
                });
                var newRating = model.Predict(unratedBook);
                y_prediction[i] = newRating;
            };

            // now find the indexes of the 3 highest ratings in the predictions
            var recommendations = y_prediction.Top(3);

            Console.WriteLine(recommendations);
            Console.ReadKey();
        }
    }
}
